
/*
 * jm.objectPrinter version 1.1
 * @Author Jan Mech
 */
 
var  jmPrinter = function() {
	/*
 	* Public Properties
 	*/ 
 
	/*
 	* Private Properties
 	*/
 
	var _maxJsObjectClasses = ["Dict", "Max", "Patcher", "Maxobj", "Wind", "Global", "Task", "Folder", "File", "Image", "Sketch", "LiveAPI", "SQLite" ];
	var _nonItaratableClasses = ["String", "Patcher", "Max", "js"]; // We dont't detect strings via  this array, but for the easy of reading we keep it here
	var _indentString = "----";
	var _config = {};

	/*
 	* Public Methods
 	*/

	/*
 	* Print any Js Object content formatted to Max Console
 	*
 	* @return void
 	*/

	var printObj = function(toPrint, indentString) {
		if(indentString != undefined) {
			_indentString = indentString;
		}
		if(toPrint != undefined) {
			post("********** [printObj Start] ********** \n");
			_iterateObject(toPrint);
			post("********** [printObj End] ********** \n");
		} else {
			error("printObj: Object is undefined. \n");
		}
		return;
	}

	/*
 	* Private methods
 	*/
 
 	/* Iterate over JS objects and print content formatted to Max console
  	*
  	* @retur void
  	*/
	var _iterateObject = function(obj, tabIndex, propertyName, firstIteration) {
		firstIteration = (firstIteration != false) ? true : false;
		tabIndex = (tabIndex == undefined) ? 0 : tabIndex;
	
		_postObjType(obj, tabIndex, propertyName);
	
		// with some objects like Patcher or jsthis we would end up in an endless iteration.
		// Therefore we Iterate them only on top level (first iteration).
	
		if(_canIterate(obj, firstIteration)) { 
			for(var i in obj) {
				if(tabIndex > 20) {
				error("printObj: Too many iterartions ( > 20) \n");
				break;
			}
				_iterateObject(obj[i], tabIndex+1, i, false);
			}
		}
	}

	/*
 	* Post object content to Max console
 	* 
 	* @return void
 	*/
	var _postObjType = function (obj, tabIndex, propertyName) {
		tabIndex = (tabIndex == undefined) ? 0 : tabIndex;
		var tabString = "";
		for (var i = 0; i< tabIndex; i++) {
			tabString = tabString + _indentString; 
		}
		post(tabString);
		if(propertyName != undefined) {
			post("\""+propertyName + "\" : ");
		}
		switch(typeof obj) {
				case "undefined" :
					post("<undefined> \n");
				break;
				
				case "object" :
					if(obj === null) {
						post("<null> \n");
					} else {
						if(Array.isArray(obj)) {
							post("Array [length: "+obj.length+"]\n");
						} else {
							// Test If it a Max specific Object
							if(_isMaxJsObject(obj)) {
								post("Max Js Object [" + _getMaxJsType(obj) + "] \n")
							} else {
								
								switch(obj.constructor.name) {
									case "String" : // When constructed with new String("foo bar") typeof will return "Object" instead of "string"
										post("String [Object | length: " +obj.length + "] "+obj+"\n");
									break;
									case "Number" : // When constructed with new Number(123) typeof will return "Object" instead of "number"
										post("Number [Object | "+_getNumberType(obj)+"] "+obj+"\n");
									break;
									case "Boolean" : // When constructed with new Boolean(true) typeof will return "Object" instead of "boolean"
										post("Boolean [Object] "+obj+"\n");
									break;
									
									default :
										post("Object ["+obj.constructor.name+"] \n");
									break;
								}	
							}
						}
					}
				break;
			
				case "boolean" :
					post("Boolean [Primitive] "+obj+"\n");
				break;
			
				case "number" :
					post("Number [Primitive | "+_getNumberType(obj)+"] "+obj+"\n");
				break;
			
				case "string" :
					post("String [Primitive | length: " +obj.length + "] "+obj+"\n");
				break;
			
				case "function" :
					post("Function \n");
				break;
				}
			return;
	}

	/*
 	* Detect if an object is specific to Max JS implementation
 	*
 	* @return boolean
 	*/
	var _isMaxJsObject = function(obj) {
		var isMaxJsObj = false;
		if(obj != undefined && obj != null) {
			isMaxJsObj = (_maxJsObjectClasses.indexOf(obj.constructor.name+"") != -1) ? true : false;
		}
		return isMaxJsObj;
	}

	/*
	 * Get Max class name for Max specific objects such as Maxobj
 	*
 	* @return mixed:  String, boolean
 	*/
	var _getMaxJsType = function(obj) {
		return (_isMaxJsObject(obj)) ? obj.constructor.name : false;
	}

	/*
	 * Some  Objects can cause infinite iteration loops (i.e. Patcher, jsthis or string (as each char inside is a string as well))
	 * This method detects them.
	 * @return boolean
	 */
	var _canIterate = function(obj, firstIteration) {
		firstIteration = (firstIteration == null) ? false : firstIteration;
		var canIterate = false;
		if(firstIteration && typeof obj != "string" && obj.constructor.name != "String") {
			canIterate = true;
		} else {
			if((obj != null && obj != undefined && typeof obj != "string") && obj.constructor.name != "String") {
				if(_isMaxJsObject && _nonItaratableClasses.indexOf(obj.constructor.name) == -1) {
					canIterate = true;
				}
			}
		}
		return canIterate;
	}

	/*
 	 * Get number type of mixed var
	 *
	 * @return mixed: String, boolean
	 */
	var _getNumberType = function(mixedVar) {
		retVal = false;
		if(_isInt(mixedVar) || _isFloat(mixedVar)) {
			retVal = (_isInt(mixedVar)) ? "Integer" : "Float";
		}
		return retVal;
	}
	var _isInt = function(mixedVar) {
		if (mixedVar.constructor.name == "Number") {
			mixedVar = mixedVar.valueOf();
		}
		return mixedVar === +mixedVar && isFinite(mixedVar) && !(mixedVar % 1);
	}
	/*
	 * Detect if a mixed var is an Float
	 *
	 * @return boolean
	 */ 
	var _isFloat = function(mixedVar) {
		if (mixedVar.constructor.name == "Number") {
			mixedVar = mixedVar.valueOf();
		}
		return +mixedVar === mixedVar && (!isFinite(mixedVar) || !! (mixedVar % 1));
	};

	/*
	 * Interface
	 */
	return {
		printObj : printObj
	};
}();


